package com.gqy.common.utils;


import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;

import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.multipart.MultipartFile;

/**
 * 文件处理工具类
 */
public class FileUtils {
    private static final Logger log = LoggerFactory.getLogger(FileUtils.class);

    /**
     * 读取文件内容
     *
     * @param file 上传的文件
     * @return 文件内容
     */
    public static String readString(MultipartFile file) throws IOException {
        return readString(file, StandardCharsets.UTF_8);
    }

    /**
     * 读取文件内容
     *
     * @param file 上传的文件
     * @param charset 字符集
     * @return 文件内容
     */
    public static String readString(MultipartFile file, Charset charset) throws IOException {
        try (InputStream inputStream = file.getInputStream()) {
            return IOUtils.toString(inputStream, charset);
        } catch (IOException e) {
            log.error("读取文件内容失败", e);
            throw new IOException("读取文件内容失败", e);
        }
    }

    /**
     * 逐行读取文件内容
     *
     * @param file 上传的文件
     * @return 文件内容
     */
    public static String readLines(MultipartFile file) throws IOException {
        return readLines(file, StandardCharsets.UTF_8);
    }

    /**
     * 逐行读取文件内容
     *
     * @param file 上传的文件
     * @param charset 字符集
     * @return 文件内容
     */
    public static String readLines(MultipartFile file, Charset charset) throws IOException {
        StringBuilder content = new StringBuilder();
        try (BufferedReader reader = new BufferedReader(new InputStreamReader(file.getInputStream(), charset))) {
            String line;
            while ((line = reader.readLine()) != null) {
                content.append(line).append(System.lineSeparator());
            }
            return content.toString();
        } catch (IOException e) {
            log.error("逐行读取文件内容失败", e);
            throw new IOException("逐行读取文件内容失败", e);
        }
    }

    /**
     * 检测文件是否为文本文件
     *
     * @param file 上传的文件
     * @return 是否为文本文件
     */
    public static boolean isTextFile(MultipartFile file) {
        String filename = file.getOriginalFilename();
        if (StringUtils.isEmpty(filename)) {
            return false;
        }

        // 检查文件扩展名
        String extension = getExtension(filename);
        if (isTextExtension(extension)) {
            return true;
        }

        // 检查文件内容
        try {
            return isTextContent(file);
        } catch (IOException e) {
            log.error("检测文件类型失败", e);
            return false;
        }
    }

    /**
     * 获取文件扩展名
     */
    public static String getExtension(String filename) {
        if (StringUtils.isEmpty(filename)) {
            return "";
        }
        int dotIndex = filename.lastIndexOf(".");
        return (dotIndex == -1) ? "" : filename.substring(dotIndex + 1).toLowerCase();
    }

    /**
     * 检查扩展名是否为文本文件
     */
    private static boolean isTextExtension(String extension) {
        String[] textExtensions = {
                "txt", "text", "json", "xml", "html", "htm", "css", "js", "md", "markdown",
                "ini", "conf", "config", "properties", "prop", "yaml", "yml",
                "sql", "java", "py", "c", "cpp", "h", "hpp", "cs", "php", "rb", "pl", "sh"
        };
        return Arrays.asList(textExtensions).contains(extension.toLowerCase());
    }

    /**
     * 通过检查文件内容来判断是否为文本文件
     */
    private static boolean isTextContent(MultipartFile file) throws IOException {
        try (InputStream is = file.getInputStream()) {
            byte[] bytes = new byte[1024];
            int read = is.read(bytes);
            if (read == -1) {
                return false;
            }

            // 检查是否包含空字节，文本文件通常不包含空字节
            for (int i = 0; i < read; i++) {
                if (bytes[i] == 0) {
                    return false;
                }
            }
            return true;
        }
    }

    /**
     * 安全的读取文件内容
     *
     * @param file 上传的文件
     * @param maxSize 最大允许大小（字节）
     * @return 文件内容
     */
    public static String readStringSafely(MultipartFile file, long maxSize) throws IOException {
        // 检查文件大小
        if (file.getSize() > maxSize) {
            throw new IOException("文件大小超过限制");
        }

        // 检查是否为文本文件
        if (!isTextFile(file)) {
            throw new IOException("不支持的文件类型");
        }

        // 读取文件内容
        return readString(file);
    }
}
