package com.gqy.document.service.impl;

import java.util.*;

import com.alibaba.fastjson2.JSONArray;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.gqy.common.core.domain.AjaxResult;
import com.gqy.common.core.text.Convert;
import com.gqy.common.exception.ServiceException;
import com.gqy.common.utils.*;
import com.gqy.document.domain.vo.PageResult;
import com.gqy.document.domain.Document;
import com.gqy.document.domain.ProjectDocument;
import com.gqy.document.mapper.DocumentMapper;
import com.gqy.document.mapper.ProjectDocumentMapper;
import com.gqy.document.service.IDocumentService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;

/**
 * 【请填写功能名称】Service业务层处理
 * 
 * @author raycos
 * @date 2024-10-30
 */
@Service
public class DcDocumentServiceImpl implements IDocumentService
{

    private static final Logger log = LoggerFactory.getLogger(DcDocumentServiceImpl.class);
    @Autowired
    private DocumentMapper documentMapper;

    @Autowired
    private ProjectDocumentMapper projectDocumentMapper;


    @Override
    @Transactional
    public AjaxResult create(HttpServletRequest request, MultipartFile file) {
        try {
            // 1. 获取表单参数
            String title = request.getParameter("title");
            String data = request.getParameter("data");
            String categoryId = request.getParameter("category_id");
            String isTemplate = request.getParameter("is_template");
            String status = request.getParameter("status");
            String projects = request.getParameter("projects");

            // 2. 参数校验
            if (StringUtils.isEmpty(title)) {
                return AjaxResult.error("文档标题不能为空");
            }

            Integer templateFlag = Convert.toInt(isTemplate, 0);
            if (templateFlag == 0 && StringUtils.isEmpty(categoryId)) {
                return AjaxResult.error("保存文档时,分类不能为空");
            }

            // 3. 检查标题是否重复
            Document existDoc = documentMapper.checkDocumentTitleUnique(title, templateFlag);
            if (StringUtils.isNotNull(existDoc)) {
                String type = templateFlag == 1 ? "模板" : "文档";
                return AjaxResult.error(String.format("%s标题 '%s' 已存在", type, title));
            }

//            // 4. 处理文件上传
//            String fileUrl = "";
//            if (file != null && !file.isEmpty()) {
//                SysFile sysFile = sysFileService.uploadFile(file);
//                if (sysFile != null) {
//                    fileUrl = sysFile.getUrl();
//                    // 如果是文本文件，尝试读取内容
//                    if (FileTypeUtils.isText(file.getOriginalFilename())) {
//                        data = FileUtils.readString(file);
//                    }
//                }
//            }

            // 5. 创建文档对象
            Document document = new Document();
            document.setTitle(title);
//            document.setData(StringUtils.isNotEmpty(data) ? data : fileUrl);
            document.setData(StringUtils.isNotEmpty(data) ? data : "");
            document.setUser_id(SecurityUtils.getUserId());
            document.setCategory_id(StringUtils.isNotEmpty(categoryId) ? Long.valueOf(categoryId) : null);
            document.setIs_template(templateFlag);
            document.setStatus(Convert.toInt(status, 5));
            document.setCreateTime(DateUtils.getNowDate());
            document.setUpdateTime(DateUtils.getNowDate());
            document.setCreateBy(SecurityUtils.getUsername());

            // 6. 保存文档
            int rows = documentMapper.insertDocument(document);
            if (rows <= 0) {
                return AjaxResult.error("创建文档失败");
            }

            // 7. 处理项目关联
            if (StringUtils.isNotEmpty(projects)) {
                try {
                    JSONArray projectArray = JSONArray.parseArray(projects);
                    List<ProjectDocument> projectDocList = new ArrayList<>();

                    for (int i = 0; i < projectArray.size(); i++) {
                        ProjectDocument pd = new ProjectDocument();
                        pd.setDoc_id(document.getId());
                        pd.setProject_id(projectArray.getLong(i));
                        pd.setStatus(5);
                        pd.setCreate_time(DateUtils.getNowDate());
                        pd.setCreate_by(SecurityUtils.getUsername());
                        projectDocList.add(pd);
                    }

                    if (!projectDocList.isEmpty()) {
                        projectDocumentMapper.batchInsertProjectDocument(projectDocList);
                    }
                } catch (Exception e) {
                    throw new ServiceException("处理项目关联失败");
                }
            }

            return AjaxResult.success("创建成功", document.getId());
        } catch (Exception e) {
            throw new ServiceException("创建文档失败：" + e.getMessage());
        }
    }

    @Override
    public PageResult pageDocument(Integer page, Integer pageSize, Document document) {
        try {
            // 1. 设置分页参数
            PageHelper.startPage(page, pageSize);

            // 2. 执行查询
            List<Map<String, Object>> list = documentMapper.pageDocument(document);

            // 3. 获取分页信息
            PageInfo<Map<String, Object>> pageInfo = new PageInfo<>(list);

            // 4. 封装并返回结果
            return new PageResult(
                    pageInfo.getPageNum(),
                    pageInfo.getPageSize(),
                    pageInfo.getPages(),
                    pageInfo.getTotal(),
                    pageInfo.getList()
            );

        } catch (Exception e) {
            log.error("查询文档列表失败", e);
            return new PageResult(page, pageSize);
        }

    }

    /**
     * 查询文档列表
     * @param page
     * @param pageSize
     * @param document
     * @return
     */
    public PageResult searchlistDoc(Integer page, Integer pageSize,Document document) {

        try {
            // 1. 设置分页参数
            PageHelper.startPage(page, pageSize);
            // 2. 执行查询
            List<Document> list = documentMapper.selectDocumentList(document);
            // 3. 获取分页信息
            PageInfo<Document> pageInfo = new PageInfo<>(list);
            // 4. 封装并返回结果
            return new PageResult(
                    pageInfo.getPageNum(),
                    pageInfo.getPageSize(),
                    pageInfo.getPages(),
                    pageInfo.getTotal(),
                    pageInfo.getList()
            );

        } catch (Exception e) {
            log.error("查询文档列表失败", e);
            return new PageResult(page, pageSize);
        }
    }


    /**
     * 查询文档详情
     *
     * @param id 文档ID
     * @return 文档信息
     */
    public Map<String, Object>  info(Long id) {
        Document doc = documentMapper.info(id);
        Map<String, Object> result = new HashMap<>();
        result.put("id", doc.getId());
        result.put("title", doc.getTitle());
        result.put("data", doc.getData());
      //  result.put("user_id", doc.getUserId());

        // 获取关联产品
        result.put("linkProduct", documentMapper.selectProductDocumentsByDocId(id));
        result.put("links", "");

        // 获取关联项目
        result.put("linkProject", documentMapper.selectProjectDocumentsByDocId(id));
        result.put("projects", "");

        result.put("category_id", doc.getCategory_id());
        result.put("createTime", doc.getCreateTime());
        result.put("status", doc.getStatus());
        result.put("is_template", doc.getIs_template());

        return result;
    }


}
